'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Datatable = exports.Col = exports.Cols = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _ColumnContainer = require('./ColumnContainer');

var _RowContainer = require('./RowContainer');

var _TotalsContainer = require('./TotalsContainer');

var _ArrowKeyNavigator = require('./ArrowKeyNavigator');

var _functions = require('../util/functions');

var _arrayDataProvider = require('./arrayDataProvider');

var _util = require('../util');

var _styles = require('./styles.css');

var _styles2 = _interopRequireDefault(_styles);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Cols = exports.Cols = function Cols() {
    return null;
};

var Col = exports.Col = function Col() {
    return null;
};
var findCols = (0, _util.findChild)(Cols);

var filterSelection = function filterSelection(_ref, _ref2) {
    var _ref4 = _slicedToArray(_ref, 2);

    var my = _ref4[0];
    var mx = _ref4[1];

    var _ref3 = _slicedToArray(_ref2, 2);

    var py = _ref3[0];
    var px = _ref3[1];
    return function (_ref5) {
        var _ref6 = _slicedToArray(_ref5, 2);

        var ny = _ref6[0];
        var nx = _ref6[1];

        if (ny > my || nx > mx || ny < 0 || nx < 0) {
            return [py, px]; //return original
        }
        return [ny, nx]; //return updated
    };
};

var getAttribute = function getAttribute(attr) {
    return function (element) {
        return (0, _functions.Id)(element.getAttribute(attr));
    };
};
var getIndex = getAttribute('data-index');
var getColIndex = getAttribute('data-col-index');

var NO_EDITOR = { index: undefined, id: undefined };

var Datatable = exports.Datatable = _react2.default.createClass({
    displayName: 'Datatable',


    propTypes: {
        children: _react.PropTypes.node,
        data: _react.PropTypes.array,
        onRowClick: _react.PropTypes.func,
        onEdit: _react.PropTypes.func
    },

    getChildContext: function getChildContext() {
        return { onEditCompleted: this.onEditCompleted };
    },

    childContextTypes: {
        onEditCompleted: _react.PropTypes.func
    },
    onEditCompleted: function onEditCompleted(_ref7) {
        var newValue = _ref7.newValue;
        var index = _ref7.index;
        var record = _ref7.record;
        var colConfig = _ref7.colConfig;
        var id = _ref7.id;
        var selection = _ref7.selection;

        if (this.props.onEdit) {
            this.props.onEdit({ newValue: newValue, index: index, record: record, colConfig: colConfig, id: id, selection: selection });
        }
        this.setState({ edited: NO_EDITOR });
    },
    getInitialState: function getInitialState() {
        var colsElement = findCols(this.props.children);
        var colConfigs = _react.Children.map(colsElement.props.children, function (child) {
            return child.props;
        });
        var dataProvider = (0, _arrayDataProvider.create)(this.props.data);
        return { selection: [0, 0], colConfigs: colConfigs, dataProvider: dataProvider, edited: NO_EDITOR, editorData: {} };
    },
    onKeyUp: function onKeyUp(e) {
        var _state = this.state;
        var colConfigs = _state.colConfigs;
        var currentEdited = _state.edited;

        var _state$selection = _slicedToArray(_state.selection, 2);

        var index = _state$selection[0];
        var col = _state$selection[1];


        if (currentEdited.index === undefined && colConfigs[col].editable === true && !(0, _ArrowKeyNavigator.isArrowKey)(e.keyCode)) {
            var edited = { index: parseInt(index), id: colConfigs[col].id, input: String.fromCharCode(e.keyCode) };
            this.setState({ edited: edited });
        }
    },
    componentDidUpdate: function componentDidUpdate(nextProps) {
        if (nextProps.data !== this.props.data) {
            var dataProvider = (0, _arrayDataProvider.create)(this.props.data);
            this.setState({ dataProvider: dataProvider });
        }
    },
    onClick: function onClick(e) {
        //determine the target of the click
        if (this.props.onRowClick) {
            this.props.onRowClick();
        }
        var target = e.target;
        var parentElement = target.parentElement;

        var index = getIndex(parentElement).map(parseInt);
        var colIndex = getColIndex(parentElement).map(parseInt);
        this.onMoveSelection([index.value, colIndex.value]);
    },
    componentDidMount: function componentDidMount() {},
    recordEdited: function recordEdited() {},
    onMoveSelection: function onMoveSelection(nextSelection) {
        var _state2 = this.state;
        var selection = _state2.selection;
        var dataProvider = _state2.dataProvider;
        var colConfigs = _state2.colConfigs;
        var edited = _state2.edited;

        var nselection = filterSelection([dataProvider.length - 1, colConfigs.length - 1], selection)(nextSelection);
        if (edited.index === undefined) {
            this.setState({ selection: nselection });
        }
    },
    render: function render() {
        var _state3 = this.state;
        var selection = _state3.selection;
        var dataProvider = _state3.dataProvider;
        var colConfigs = _state3.colConfigs;
        var edited = _state3.edited;


        return _react2.default.createElement(
            'div',
            { onClick: this.onClick, onKeyUp: this.onKeyUp, className: _styles2.default['rui-dg'] },
            _react2.default.createElement(
                _ArrowKeyNavigator.ArrowKeyNavigator,
                {
                    onMoveSelection: this.onMoveSelection, selection: selection, className: _styles2.default['rui-dg-container'] },
                _react2.default.createElement(_ColumnContainer.ColumnContainer, { selection: selection, data: colConfigs }),
                _react2.default.createElement(_RowContainer.RowContainer, { selection: selection, edited: edited, colConfigs: colConfigs, dataProvider: dataProvider }),
                _react2.default.createElement(_TotalsContainer.TotalsContainer, { selection: selection, colConfigs: colConfigs, dataProvider: dataProvider })
            )
        );
    }
});