'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
var create = exports.create = function create() {
    var backingArray = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];


    if (!Array.isArray(backingArray)) {
        throw new Error('argument must be an array');
    }

    var api = {
        get length() {
            return backingArray.length;
        },

        recordAt: function recordAt(i) {
            return backingArray[i];
        },


        /**
         * Iterate on the records starting with startIdx or 0 (default)
         * and endIdx or length -1;
         *
         * mapper is passed the following parameters
         *
         * record, index, endIdx, dataProvider
         *
         *
         * @param mapper
         * @param start
         * @param end
         */
        map: function map(mapper, startIdx, endIdx) {
            var start = startIdx || 0;
            var end = endIdx || backingArray.length;
            var result = [];

            for (var i = start; i < end; i++) {
                result.push(mapper(backingArray[i], i, endIdx, api));
            }
            return result;
        },
        reduce: function reduce(reducer, initialVal) {
            if (!initialVal) {
                return backingArray.reduce(reducer);
            }
            return backingArray.reduce(reducer, initialVal);
        }
    };
    return api;
};

exports.default = create;